@@ -14,6 +14,7 @@ const intercom = new EventEmitter();
 const SCHED_NONE = 1;
 const SCHED_RR = 2;
 const { isLegalPort } = require('internal/net');
+const [ minPort, maxPort ] = [ 1024, 65535 ];
 
 module.exports = cluster;
 
@@ -119,6 +120,8 @@ function createWorkerProcess(id, env) {
       }
     } else {
       inspectPort = process.debugPort + debugPortOffset;
+      if (inspectPort > maxPort)
+        inspectPort = inspectPort - maxPort + minPort - 1;
       debugPortOffset++;
     }
 
