@@ -1,4 +1,3 @@
-#undef NDEBUG
 #include <assert.h>
 #include <stdlib.h>
 #include <node.h>
@@ -33,11 +32,11 @@ static void sanity_check(void) {
   assert(at_exit_cb2_called == 2);
 }
 
-void init(Local<Object> target) {
-  AtExit(at_exit_cb1, target->CreationContext()->GetIsolate());
+void init(Local<Object> exports) {
+  AtExit(at_exit_cb1, exports->GetIsolate());
   AtExit(at_exit_cb2, cookie);
   AtExit(at_exit_cb2, cookie);
   atexit(sanity_check);
 }
 
-NODE_MODULE(binding, init);
+NODE_MODULE(NODE_GYP_MODULE_NAME, init)
